@@ -98,6 +98,8 @@ pub struct TabWriter<W> {
     minwidth: usize,
     padding: usize,
     alignment: Alignment,
+    ansi: bool,
+    tab_indent: bool,
 }
 
 /// `Alignment` represents how a `TabWriter` should align text within its cell.
@@ -128,13 +130,15 @@ impl<W: io::Write> TabWriter<W> {
     /// write to the given writer.
     pub fn new(w: W) -> TabWriter<W> {
         TabWriter {
-            w: w,
+            w,
             buf: io::Cursor::new(Vec::with_capacity(1024)),
             lines: vec![vec![]],
             curcell: Cell::new(0),
             minwidth: 2,
             padding: 2,
             alignment: Alignment::Left,
+            ansi: cfg!(feature = "ansi_formatting"),
+            tab_indent: false,
         }
     }
 
@@ -167,6 +171,24 @@ impl<W: io::Write> TabWriter<W> {
         self
     }
 
+    /// Ignore ANSI escape codes when computing the number of display columns.
+    ///
+    /// This is disabled by default. (But is enabled by default when the
+    /// deprecated `ansi_formatting` crate feature is enabled.)
+    pub fn ansi(mut self, yes: bool) -> TabWriter<W> {
+        self.ansi = yes;
+        self
+    }
+
+    /// Always use tabs for indentation columns (i.e., padding of
+    /// leading empty cells on the left).
+    ///
+    /// This is disabled by default.
+    pub fn tab_indent(mut self, yes: bool) -> TabWriter<W> {
+        self.tab_indent = yes;
+        self
+    }
+
     /// Unwraps this `TabWriter`, returning the underlying writer.
     ///
     /// This internal buffer is flushed before returning the writer. If the
@@ -199,7 +221,11 @@ impl<W: io::Write> TabWriter<W> {
         let mut curcell = Cell::new(self.buf.position() as usize);
         mem::swap(&mut self.curcell, &mut curcell);
 
-        curcell.update_width(&self.buf.get_ref());
+        if self.ansi {
+            curcell.update_width(&self.buf.get_ref(), count_columns_ansi);
+        } else {
+            curcell.update_width(&self.buf.get_ref(), count_columns_noansi);
+        }
         self.curline_mut().push(curcell);
     }
 
@@ -218,12 +244,16 @@ impl<W: io::Write> TabWriter<W> {
 
 impl Cell {
     fn new(start: usize) -> Cell {
-        Cell { start: start, width: 0, size: 0 }
+        Cell { start, width: 0, size: 0 }
     }
 
-    fn update_width(&mut self, buf: &[u8]) {
+    fn update_width(
+        &mut self,
+        buf: &[u8],
+        count_columns: impl Fn(&[u8]) -> usize,
+    ) {
         let end = self.start + self.size;
-        self.width = display_columns(&buf[self.start..end]);
+        self.width = count_columns(&buf[self.start..end]);
     }
 }
 
@@ -276,6 +306,8 @@ impl<W: io::Write> io::Write for TabWriter<W> {
             } else {
                 first = false
             }
+
+            let mut use_tabs = self.tab_indent;
             for (i, cell) in line.iter().enumerate() {
                 let bytes =
                     &self.buf.get_ref()[cell.start..cell.start + cell.size];
@@ -284,6 +316,12 @@ impl<W: io::Write> io::Write for TabWriter<W> {
                     assert_eq!(i, line.len() - 1);
                     self.w.write_all(bytes)?;
                 } else {
+                    if use_tabs && cell.size == 0 {
+                        write!(&mut self.w, "\t")?;
+                        continue;
+                    }
+                    use_tabs = false;
+
                     assert!(widths[i] >= cell.width);
                     let extra_space = widths[i] - cell.width;
                     let (left_spaces, mut right_spaces) = match self.alignment
@@ -338,6 +376,7 @@ impl<W> fmt::Display for IntoInnerError<W> {
 }
 
 impl<W: ::std::any::Any> error::Error for IntoInnerError<W> {
+    #[allow(deprecated)]
     fn description(&self) -> &str {
         self.error().description()
     }
@@ -378,8 +417,7 @@ fn cell_widths(lines: &Vec<Vec<Cell>>, minwidth: usize) -> Vec<Vec<usize>> {
     ws
 }
 
-#[cfg(not(feature = "ansi_formatting"))]
-fn display_columns(bytes: &[u8]) -> usize {
+fn count_columns_noansi(bytes: &[u8]) -> usize {
     use unicode_width::UnicodeWidthChar;
 
     // If we have a Unicode string, then attempt to guess the number of
@@ -393,8 +431,7 @@ fn display_columns(bytes: &[u8]) -> usize {
     }
 }
 
-#[cfg(feature = "ansi_formatting")]
-fn display_columns(bytes: &[u8]) -> usize {
+fn count_columns_ansi(bytes: &[u8]) -> usize {
     use unicode_width::UnicodeWidthChar;
 
     // If we have a Unicode string, then attempt to guess the number of
@@ -408,12 +445,34 @@ fn display_columns(bytes: &[u8]) -> usize {
     }
 }
 
-#[cfg(feature = "ansi_formatting")]
 fn strip_formatting<'t>(input: &'t str) -> std::borrow::Cow<'t, str> {
-    use regex::Regex;
-
-    lazy_static::lazy_static! {
-        static ref RE: Regex = Regex::new(r#"\x1B\[.+?m"#).unwrap();
+    let mut escapes = find_ansi_escapes(input).peekable();
+    if escapes.peek().is_none() {
+        return std::borrow::Cow::Borrowed(input);
     }
-    RE.replace_all(input, "")
+    let mut without_escapes = String::with_capacity(input.len());
+    let mut last_end = 0;
+    for mat in escapes {
+        without_escapes.push_str(&input[last_end..mat.start]);
+        last_end = mat.end;
+    }
+    without_escapes.push_str(&input[last_end..]);
+    std::borrow::Cow::Owned(without_escapes)
+}
+
+fn find_ansi_escapes<'t>(
+    input: &'t str,
+) -> impl Iterator<Item = std::ops::Range<usize>> + 't {
+    const ESCAPE_PREFIX: &str = "\x1B[";
+    let mut last_end = 0;
+    std::iter::from_fn(move || {
+        let start = last_end
+            + input[last_end..].match_indices(ESCAPE_PREFIX).next()?.0;
+        let after_prefix = start + ESCAPE_PREFIX.len();
+        let end = after_prefix
+            + input[after_prefix..].match_indices('m').next()?.0
+            + 1;
+        last_end = end;
+        Some(start..end)
+    })
 }
